@@ -6,21 +6,24 @@ import sys
 import re
 import logging
 from collections import OrderedDict
+from time import sleep
 
 from .config import Config
 from .utils import (safe_makedirs, value_interpolate, construct_proxy_commands,
                     shellquotemultiple)
+from . import __version__
 
 
 class AdvancedSshConfig(object):
 
     def __init__(self, hostname=None, port=None, configfiles=None,
                  verbose=False, dry_run=False, proxy_type='nc',
-                 timeout=180):
+                 timeout=180, use_python_socket=False):
 
         self.verbose, self.dry_run = verbose, dry_run
         self.hostname, self.port = hostname, port
         self.proxy_type, self.timeout = proxy_type, timeout
+        self.user_python_socket = use_python_socket
 
         self.log = logging.getLogger('')
 
@@ -32,9 +35,6 @@ class AdvancedSshConfig(object):
                 ]
         self.config = Config(configfiles=configfiles)
 
-    def debug(self, string=None):
-        self.log.debug(string and string or '')
-
     @property
     def controlpath_dir(self):
         controlpath = self.config.get('controlpath',
@@ -54,7 +54,7 @@ class AdvancedSshConfig(object):
             if re.match(sect, self.hostname):
                 section = sect
 
-        self.debug('section \'{}\' '.format(section))
+        logging.debug('section "{}" '.format(section))
 
         # Parse special routing
         path = self.hostname.split('/')
@@ -83,7 +83,9 @@ class AdvancedSshConfig(object):
                 self.config.parser.set(section, options[key], value)
                 args[key] = value
 
-            self.debug('get (-%-1s) %-12s : %s' % (key, options[key], value))
+            logging.debug('get (-%-1s) %-12s : %s' % (key,
+                                                      options[key],
+                                                      value))
             if value:
                 args[key] = value
 
@@ -92,27 +94,18 @@ class AdvancedSshConfig(object):
             self.write_sshconfig()
             self.log.debug('Config updated. Need to restart SSH!?')
 
-        self.debug('args: {}'.format(args))
-        self.debug()
+        logging.debug('args: {}'.format(args))
 
-        self.debug('hostname    : {}'.format(self.hostname))
-        self.debug('port        : {}'.format(self.port))
-        self.debug('path        : {}'.format(path))
-        self.debug('path[0]     : {}'.format(path[0]))
-        self.debug('path[1:]    : {}'.format(path[1:]))
-        self.debug('args        : {}'.format(args))
-
-        self.debug()
         routing['verbose'] = self.verbose
         routing['proxy_type'] = self.proxy_type
-        routing['gateways'] = self.config.get('Gateways', path[-1], 'direct')
-        routing['comment'] = self.config.get('comment', path[-1], None)
-        routing['reallocalcommand'] = self.config.get('RealLocalCommand',
-                                                      path[-1], '')
-        for key in ('gateways', 'reallocalcommand'):
-            routing[key] = routing[key].strip().split(' ')
-        self.debug('reallocalcommand : {}'.format(routing['reallocalcommand']))
-        self.debug('gateways         : {}'.format(', '.join(['gateways'])))
+        for special_key in ('comment', 'password', 'gateways',
+                            'reallocalcommand'):
+            routing[special_key] = self.config.get(special_key, path[-1], None)
+        if not routing['gateways']:
+            routing['gateways'] = ['direct']
+        else:
+            routing['gateways'] = routing['gateways'].split(' ')
+
         routing['gateway_route'] = path[1:]
         routing['hostname'] = args['h']
         #routing['args'] = args
@@ -123,11 +116,9 @@ class AdvancedSshConfig(object):
             routing['port'] = 22
         routing['proxy_commands'] = construct_proxy_commands(routing)
 
-        self.debug()
-        self.debug('Routing:')
+        logging.debug('Routing:')
         for key, value in routing.iteritems():
-            self.debug('  {0}: {1}'.format(key, value))
-        self.debug()
+            logging.debug('  {0}: {1}'.format(key, value))
 
         return routing
 
@@ -135,29 +126,57 @@ class AdvancedSshConfig(object):
         for gateway in routing['gateways']:
             if gateway != 'direct':
                 routing['gateway_route'] += [gateway]
+                logging.info('Using gateway: '
+                             '{}'.format(routing['gateway_route']))
+            else:
+                logging.info('Direct connection')
             cmd = []
             if len(routing['gateway_route']):
                 cmd += ['ssh', '/'.join(routing['gateway_route'])]
                 cmd.append(shellquotemultiple(routing['proxy_commands']))
+                logging.info('cmd: {}'.format(cmd))
             else:
-                cmd += routing['proxy_commands'][0]
+                cmd = routing['proxy_commands'][0]
 
-            self.debug('cmd         : {}'.format(cmd))
-            self.debug('================')
-            self.debug()
+            logging.info('Connection command {}'.format(map(str, cmd)))
 
             if not self.dry_run:
-                comment = routing.get('comment', None)
-                if comment:
-                    sys.stderr.write('{}\n'.format(comment))
-                ssh_process = subprocess.Popen(map(str, cmd))
-                rlc_process = None
-                if len(routing['reallocalcommand'][0]):
-                    rlc_process = subprocess.Popen(routing['reallocalcommand'])
-                if ssh_process.wait() != 0:
-                    self.log.critical('There were some errors')
-                if rlc_process is not None:
-                    rlc_process.kill()
+                self.connect_once(routing, cmd)
+
+    def connect_once(self, routing, cmd):
+        comment = routing.get('comment', None)
+        if comment:
+            sys.stderr.write('{}\n'.format('\n'.join(comment)))
+
+        password = routing.get('password', None)
+        if password:
+            sys.stderr.write('password: {}\n'.format(password))
+
+        rlc_process = None
+        if routing['reallocalcommand']:
+            logging.info('Executing localcommand: '
+                         '{}'.format(routing['reallocalcommand']))
+            rlc_cmd = ['/bin/sh', '-c', routing['reallocalcommand']]
+            rlc_process = subprocess.Popen(rlc_cmd,
+                                           stdout=sys.stderr,
+                                           stderr=sys.stderr)
+            sleep(.1)
+
+        if self.user_python_socket \
+                and not len(routing['gateway_route']):
+            logging.info('Using Python socket')
+            from .network import Socket
+            socket = Socket(routing['hostname'], routing['port'])
+            socket.run()
+        else:
+            logging.info('Using ProxyCommand')
+            proxy_process = subprocess.Popen(map(str, cmd))
+            if proxy_process.wait() != 0:
+                self.log.critical('There were some errors')
+
+        if rlc_process is not None:
+            print(rlc_process)
+            rlc_process.kill()
 
     def write_sshconfig(self, filename='~/.ssh/config'):
         config = self.build_sshconfig()
@@ -168,6 +187,9 @@ class AdvancedSshConfig(object):
     def build_sshconfig(self):
         config = []
 
+        config.append('# assh version: {}'.format(__version__))
+        config.append('')
+
         hosts = self.prepare_sshconfig()
         od = OrderedDict(sorted(hosts.items()))
         for entry in od.values():
